/*************************************************************************************/
/*                                                                                   */
/* SUDOKU SOLVER AND GENERATOR                                                       */
/*                                                                                   */
/* OVERVIEW OF ALGORITHM:                                                            */
/*                                                                                   */
/* This solver "library" makes use of what may be called as the constraints of a     */
/* sudoku grid to solve most of the relatively simple ones successfully. As a        */
/* preliminary, it can be stated that out of the 17,445 puzzles stored in the file   */
/* subig20.txt (which we found on some forum on the net) in the test data folder     */
/* the 1st week's algorithm could finish off more than 10,200 puzzles while also     */
/* trying to solve the other 7,200 or so puzzles - all in a mere 43 seconds on the   */
/* test pc - an Ubuntu virtual machine with a Windows 7 host on a Core i7 processor  */
/* - which can be safely deemed to be fairly above mainstream. On the OSL computers  */
/* the algorithm finished (of course, with the same results) within 2 minutes. At    */
/* that point, however, it could solve none of the top95.txt puzzles. So we ended    */
/* up making a backtracking algorithm by recursively calling the solver function     */
/* and using a guessing algorithm. Funnily though, on optimizing the guessing        */
/* algorithm to select the cell with lowest number of possibilities (i.e. >=2) and   */
/* guess the first valid possibility, when the logic is not able to proceed further  */
/* actually increased the average number of guesses (or recursions) required to      */
/* solve the puzzles in the top95.txt file! This was slightly surprising (Kandarp    */
/* can answer the why if required) as it became much faster in case of the easy      */
/* puzzles. Finally, we ended up with the "old" guessing technique (as explained     */
/* later) as we felt that guessing will be required to be really quick only for the  */
/* harder ones where the new algorithm was substantially slower!                     */
/*                                                                                   */
/*************************************************************************************/

using namespace std;

/********************************/
/******* Global Variables *******/
/********************************/

int recursions=0; // counts the number of total guesses made while solving the sudoku
int correctguesses=0; // counts the number of correct guesses required to solve the sudoku (recursive depth)
int failedguesses=0; // counts the number of incorrect guesses made while solving the sudoku
int logictimes=0; // counts the number of times the logic() algorithm executed, other than the first iteration, when called once
int diffscore=0; // assigns a rudimentary (integral) level of difficulty to a sudoku puzzle
int multiple=0; // tracks the number of solutions possible for a given sudoku
int no_guessing_reqd=1; // user choice whether the sudoku generated should require any advance techniques/guesswork or not; default choice is yes <=> 1
int is_puzzle_symmetric=1; // user choice whether the sudoku generated should be rotationally symmetric in appearance or not; default choice is yes <=> 1
int fnflag=0; // tracks whether the generate() algorithm is solving an almost blank puzzle or not to first have a filled grid and hence whether to suppress checking mechanisms of uniqueness, etc. and hide corresponding user alerts or not; 1 => generate() is using the solver for the almost blank grid, 0 => solve() or hideblanks() are using the solver for a valid question grid
int difflevel=1; // assigns the difficulty level as explained in settings window with default choice being yes <=> 1
int solve_button=0; // tracks if solve() button is invoked and hence whether to display an error message if uniqueness tests fail or not
char question_linegrid[82],solution_linegrid[82]; // these character arrays store the generated/loaded question grids and first solution (if any) of any grid for easy access without parameter passing

/********************************/
/****** Class Declaration *******/
/********************************/

class sudoku{

	public:

	/********************************/
	/****** Variables of Class ******/
	/********************************/

	int grid[9][9]; // stores the actual values present on the grid in each cell with blanks as zeroes
	int posbl[9][9][9]; // stores all the remaining possibilities for each cell of the grid in the 3d array
	int np[9][9]; // stores the number of remaining valid possibilities for each cell

	/*********************************************/
	/*** Declaration of Class Member Functions ***/
	/*********************************************/

	// terminal-display functions
	
	void printsudoku();
	
	/****************************************************************************/
	/*                                                                          */
	/* Prints the sudoku as a single line on the terminal (for debugging).      */
	/*                                                                          */
	/****************************************************************************/
	
	void decorated_sudoku();
	
	/****************************************************************************/
	/*                                                                          */
	/* Prints a relative more polished appearance of the sudoku on terminal.    */
	/*                                                                          */
	/****************************************************************************/
	
	// correctness, uniqueness and other checks
	
	int is_grid_solved();
	
	/****************************************************************************/
	/*                                                                          */
	/* Checks whether or not the grid is solved based on the value of np at a   */
	/* cell being -5 everywhere and grid value not being zero.                  */
	/*                                                                          */
	/****************************************************************************/
	
	int is_pattern_unsolvable();
	
	/****************************************************************************/
	/*                                                                          */
	/* If np at any cell becomes 0 => the grid is unsolvable as min. number of  */
	/* valid possibilities at any position for a correct question is 1.         */
	/*                                                                          */
	/****************************************************************************/
	
	int checkbasicuniqueness();
	
	/****************************************************************************/
	/*                                                                          */
	/* It is proved that at least 8 unique numbers must be present on the grid  */
	/* and also that till now no sudoku below 17 givens has been found to have  */
	/* a unique solution. These facts/hypothesis are checked for in this fn.    */
	/*                                                                          */
	/****************************************************************************/
	
	int presentnoscheck();
	
	/****************************************************************************/
	/*                                                                          */
	/* Counts how many numbers from 1 to 9 are present in the grid.             */
	/*                                                                          */
	/****************************************************************************/
	
	int checkblanks();
	
	/****************************************************************************/
	/*                                                                          */
	/* Simply counts the number of blanks in the given grid.                    */
	/*                                                                          */
	/****************************************************************************/
	
	// main solving algorithms
	
	void logic();
	
	/****************************************************************************/
	/*                                                                          */
	/* Executes the next four functions to eliminate possibilities abd plug in  */
	/* any found values. Repeats itself if a new value is found.                */
	/*                                                                          */
	/****************************************************************************/
	
	void checkrc(int,int);
	
	/****************************************************************************/
	/*                                                                          */
	/* If a cell [i][j] have value k+1 then k+1 cannot occur in any cell in the */
	/* same row and column as that cell. This implements removal of such posbl. */
	/*                                                                          */
	/****************************************************************************/
	
	void rcnootherposbl(int,int,int);
	
	/****************************************************************************/
	/*                                                                          */
	/* If k is a value possible in only one cell in a row or column then it is  */
	/* implicitly implied that that cell must have number k plugged in there.   */
	/*                                                                          */
	/****************************************************************************/
	
	void checkbox(int,int,int,int);
	
	/****************************************************************************/
	/*                                                                          */
	/* If a cell [i][j] have value k+1 then k+1 cannot occur in any cell in     */
	/* the square box or sub-grid containing that cell.                         */
	/*                                                                          */
	/****************************************************************************/
	
	void nootherposbl(int,int,int,int,int);
	
	/****************************************************************************/
	/*                                                                          */
	/* If k is a value possible in only one cell in a square box then it is     */
	/* implicitly implied that that cell must have number k plugged in there.   */
	/*                                                                          */
	/****************************************************************************/
	
	void updatedependencies(int,int,int);
	
	/****************************************************************************/
	/*                                                                          */
	/* Updates the possibilities in the cells in the same row, cloumn and box   */
	/* as the cell which is just filled by the algorithm/user.                  */
	/*                                                                          */
	/****************************************************************************/
	
	int guess();
	
	/****************************************************************************/
	/*                                                                          */
	/* The recusive guessing algorithm. If called, finds the first blank cell,  */
	/* guesses a valid possibility in a tmp sudoku object and proceed to solve  */
	/* it further ahead. If it fails, it tries the next value possible. If for  */
	/* a cell all values are exhausted, returns that the grid is unsolvable.    */
	/*                                                                          */
	/****************************************************************************/
	
	// central function calling solving algorithms
	
	int solved();
	
	/****************************************************************************/
	/*                                                                          */
	/* Executes the logic, performs some checks, calls guess() if necessary.    */
	/*                                                                          */
	/****************************************************************************/
	
	// generator-specific functions
	
	void initrandobj();
	
	/****************************************************************************/
	/*                                                                          */
	/* Initializes a blank grid with 9 random numbers in appropriate positions  */
	/* and then solves the hardly-filled grid by brute-force and uses the first */
	/* solution obtained as the solution of the generated sudoku.               */
	/*                                                                          */
	/****************************************************************************/
	
	int hideblanks(int);
	
	/****************************************************************************/
	/*                                                                          */
	/* Removes random numbers from the filled grid generated by initrandobj()   */
	/* each time checking for single solutions and validity of the grid till    */
	/* desired number of blanks is reached. Makes symmetrical or asymmetrical   */
	/* sudoku grids.                                                            */
	/*                                                                          */
	/****************************************************************************/
	
	int randnumblanks();
	
	/****************************************************************************/
	/*                                                                          */
	/* Randomizes the number of blanks to be created on the generated puzzle.   */
	/*                                                                          */
	/****************************************************************************/
	
	// solver-specific functions
	
	void init_for_solver(char[]);
	
	/****************************************************************************/
	/*                                                                          */
	/* Initializes the variables for the solve() function to use.               */
	/*                                                                          */
	/****************************************************************************/
	
	void printactivity(int,int,int,int);
	
	/****************************************************************************/
	/*                                                                          */
	/* Prints the activity i.e. which value filled which cell by which method.  */
	/*                                                                          */
	/****************************************************************************/
	
	void init_current_obj();
	
	/****************************************************************************/
	/*                                                                          */
	/* Puts a blank grid onto the object and clears all variable to balnks.     */
	/*                                                                          */
	/****************************************************************************/
	
	// saving and loading functionality
	
	void line_to_grid(char[]);
	
	/****************************************************************************/
	/*                                                                          */
	/* Converts a line sudoku representation to a grid i.e. matrix.             */
	/*                                                                          */
	/****************************************************************************/
	
	void grid_to_line(int[][9],char[]);
	
	/****************************************************************************/
	/*                                                                          */
	/* Converts a grid sudoku representation to a line row by row.              */
	/*                                                                          */
	/****************************************************************************/

};

/********************************/
/*** Declaration of Functions ***/
/********************************/

// functions called by the GUI after some initialization

void try_to_solve(char[]);

/****************************************************************************/
/*                                                                          */
/* Does necessary conversion of grids/lines and calls the solver function.  */
/*                                                                          */
/****************************************************************************/
	
void generate();

/****************************************************************************/
/*                                                                          */
/* Calls all the necessary fns. in sequential order to generate a new grid. */
/*                                                                          */
/****************************************************************************/
	
// alert window implementation

void alert(char[],char[]);

/****************************************************************************/
/*                                                                          */
/* Displays an alert window to inform the user about any message/error.     */
/*                                                                          */
/****************************************************************************/

/********************************************/
/*** Definition of Class Member Functions ***/
/********************************************/

// terminal-display functions
void sudoku::printsudoku(){
	for(int i=0;i<9;i++){
		for(int j=0;j<9;j++){
			cout<<grid[i][j];
		}
	}
	cout<<endl;
}
void sudoku::decorated_sudoku(){
	cout<<"#-----+-----+-----#"<<endl;
	for(int i=0;i<9;i++){
		cout<<"| ";
		for(int j=0;j<3;j++){
			cout<<grid[i][j];
		}
		cout<<" | ";
		for(int j=3;j<6;j++){
			cout<<grid[i][j];
		}
		cout<<" | ";
		for(int j=6;j<9;j++){
			cout<<grid[i][j];
		}
		cout<<" |";
		cout<<endl;
		if(i==2 || i==5){
			cout<<"|-----+-----+-----|"<<endl;
		}
	}
	cout<<"#-----+-----+-----#"<<endl;
}

// correctness, uniqueness and other checks
int sudoku::is_grid_solved(){
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		if(grid[i][j]==0){
			return 0;
		}
	}
	}
	return 1;
}
int sudoku::is_pattern_unsolvable(){
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		if(!(np[i][j]>0 || np[i][j]==-5)){
			return (1); // Failed Recursion or Unsolvable Sudoku!
		}
	}
	}
	if(checkbasicuniqueness()==0 && fnflag==0){
		if(solve_button==1){
			char alert_window_text1[500]="Invalid sudoku entered!",alert_window_text2[500]="(Uniqueness Tests Failed)";
			alert(alert_window_text1,alert_window_text2);
		}
		return (1);
	}
	return (0);
}
int sudoku::checkbasicuniqueness(){
	if(checkblanks()>64 || presentnoscheck()<8){
		return (0);
	}
	return (1);
}
int sudoku::presentnoscheck(){
	int present[9]={0,0,0,0,0,0,0,0,0},presentnos=0;
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		if(grid[i][j]!=0){
			if(present[(grid[i][j]-1)]!=1){
				present[(grid[i][j]-1)]=1;
				presentnos++;
			}
		}
	}
	}
	return presentnos;
}
int sudoku::checkblanks(){
	int blanks=0;
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		if(grid[i][j]==0){
			blanks++;
		}
	}
	}
	return (blanks);
}

// main solving algorithms
void sudoku::logic(){
	logictimes--;
	int iterations=0,checktcells=checkblanks();
	while(checkblanks()!=0 && iterations<2){
		logictimes++;
		for(int i=0;i<9;i++){
		for(int j=0;j<9;j++){
			if(grid[i][j]==0){
			checkrc(i,j);
			checkbox(3*(int)(i/3),3*(int)(j/3),i,j);
			if(np[i][j]==1){
				for(int k=0;k<9;k++){
					if(posbl[i][j][k]>0){
						grid[i][j]=posbl[i][j][k]; np[i][j]=-5;
						updatedependencies(i,j,k);
						//printactivity(i,j,grid[i][j],1);
						break;
					}
				}
			}
				if(grid[i][j]==0){
					rcnootherposbl(i,j,iterations);
				}
				if(grid[i][j]==0){
					nootherposbl(3*(int)(i/3),3*(int)(j/3),i,j,iterations);
				}
			}
		}
		}
		if(checktcells==checkblanks()){
			iterations++;
			logictimes+=2;
		}
		else{
			iterations=0;
			checktcells=checkblanks();
		}
	}
}
void sudoku::checkrc(int i,int j){
	for(int k=0;k<9;k++){
		if(posbl[i][j][k]!=-1){
			for(int l=0;l<9;l++){
				if((posbl[i][j][k]==grid[i][l]) || (posbl[i][j][k]==grid[l][j])){
					posbl[i][j][k]=-1; np[i][j]--;
				}
			}
		}
	}
}
void sudoku::rcnootherposbl(int i,int j,int itr){
	for(int k=0;k<9;k++){
		if(posbl[i][j][k]>0){
			// row done now
			int flag1=0,flag2=1;
			for(int r=0;r<9;r++){
				if(grid[i][r]==0 && !(r==j)){
					if(posbl[i][r][k]==-1){
						flag1=1;	// flag1=1 => impossible in blank peers and not present in 3x3 box
					}
					else{			
						flag2=0;	// flag2=0 => unknown state of possibility
					}
				}
			}
			if(flag1==1 && flag2==1){
				grid[i][j]=posbl[i][j][k]; np[i][j]=-5;
				updatedependencies(i,j,k);
				//printactivity(i,j,grid[i][j],2);
				break;
			}
			// column done now
			flag1=0;flag2=1;
			for(int c=0;c<9;c++){
				if(grid[c][j]==0 && !(c==i)){
					if(posbl[c][j][k]==-1){
						flag1=1;	// flag1=1 => impossible in blank peers and not present in 3x3 box
					}
					else{			
						flag2=0;	// flag2=0 => unknown state of possibility
					}
				}
			}
			if(flag1==1 && flag2==1){
				grid[i][j]=posbl[i][j][k]; np[i][j]=-5;
				updatedependencies(i,j,k);
				//printactivity(i,j,grid[i][j],2);
				break;
			}
		}
	}
}
void sudoku::checkbox(int istart,int jstart,int i,int j){
	for(int t=0;t<9;t++){
		if(posbl[i][j][t]!=-1){
			for(int k=istart;k<istart+3 && (posbl[i][j][t]!=-1);k++){
			for(int l=jstart;l<jstart+3 && (posbl[i][j][t]!=-1);l++){
				if(grid[k][l]==posbl[i][j][t]){
					posbl[i][j][t]=-1; np[i][j]--;
				}
			}
			}
		}
	}
}
void sudoku::nootherposbl(int istart,int jstart,int i,int j,int itr){
	for(int k=0;k<9;k++){
		if(posbl[i][j][k]>0){
			int flag1=0,flag2=1;
			int x=i/3,y=j/3;
			istart=x*3;jstart=y*3;
			for(int m=istart;m<istart+3;m++){
			for(int n=jstart;n<jstart+3;n++){
				if(grid[m][n]==0 && !(m==i && n==j)){
					if(posbl[m][n][k]==-1){
					flag1=1;	// flag1=1 => impossible in blank peers and not present in 3x3 box
					}
					else{			
					flag2=0;	// flag2=0 => unknown state of possibility
					}
				}
			}
			}
			if(flag1==1 && flag2==1){
				grid[i][j]=posbl[i][j][k]; np[i][j]=-5;
				updatedependencies(i,j,k);
				//printactivity(i,j,grid[i][j],3);
				break;
			}
		}
	}
}
void sudoku::updatedependencies(int i,int j,int k){
	for(int x=0;x<9;x++){
		if(posbl[x][j][k]!=-1){
		posbl[x][j][k]=-1;
			if(grid[x][j]==0){
				np[x][j]--;
			}
		}
		if(posbl[i][x][k]!=-1){
		posbl[i][x][k]=-1;
			if(grid[i][x]==0){
				np[i][x]--;
			}
		}
	}
	for(int x=3*(int)(i/3);x<3*(int)(i/3)+3;x++){
	for(int y=3*(int)(j/3);y<3*(int)(j/3)+3;y++){
		if(!(x==i && y==j) && posbl[x][y][k]!=-1){
			posbl[x][y][k]=-1;
			if(grid[x][y]==0){
				np[x][y]--;
			}
		}
	}
	}
}
int sudoku::guess(){
	for(int i=fnflag;i<9;i++){
	for(int j=0;j<9;j++){
		if(grid[i][j]==0){
			for(int k=0;k<9;k++){
				if(posbl[i][j][k]!=-1){
					sudoku tmp=*this;
					tmp.grid[i][j]=posbl[i][j][k];
					tmp.np[i][j]=-5;
					if(fnflag==0){
						recursions++;
					}
					tmp.updatedependencies(i,j,k);
					if(tmp.solved()){
						if(fnflag==1 || multiple==500){
							*this=tmp;
							return 1;
						}
					}
				}
			}
			return 0;
		}
	}
	}
	return 0;
}

// central function calling solving algorithms
int sudoku::solved(){
	int guessresult;
	if(is_pattern_unsolvable()){
		return 0;
	}
	logic();
	if(is_pattern_unsolvable()){
		return 0;
	}
	if(fnflag==1){
		if(is_grid_solved()){
			return 1;
		}
		return guess();
	}
	else if(fnflag==0){
		if(is_grid_solved()){
			if(multiple>0){
				multiple++;
				return 1;
			}
			else if(multiple==0){
				grid_to_line(grid,solution_linegrid);
				multiple++;
				return (1);
			}
		}
		guessresult=guess();
		if(guessresult=1){
			correctguesses++;
		}
		else if(guessresult=0){
			failedguesses++;
		}
		return guessresult;
	}
}

// generator-specific functions
void sudoku::initrandobj(){
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		grid[i][j]=0;
		np[i][j]=9;
			for(int k=0;k<9;k++){
				posbl[i][j][k]=k+1;
			}
	}
	}
	srand(time(NULL));
	for(int i=0,j=0;j<3;j++){
		while(posbl[i][j][(grid[i][j]=(rand()%9+1))-1]==-1);
		np[i][j]=-5;
		updatedependencies(i,j,grid[i][j]-1);
	}
	for(int i=3,j=3;j<6;j++){
		while(posbl[i][j][(grid[i][j]=(rand()%9+1))-1]==-1);
		np[i][j]=-5;
		updatedependencies(i,j,grid[i][j]-1);
	}
	for(int i=6,j=6;j<9;j++){
		while(posbl[i][j][(grid[i][j]=(rand()%9+1))-1]==-1);
		np[i][j]=-5;
		updatedependencies(i,j,grid[i][j]-1);
	}
}
int sudoku::hideblanks(int numblanks){
	fnflag=1;
	sudoku foruser=*this;
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		if(grid[i][j]!=0){
			for(int k=0;k<9;k++){
				foruser.posbl[i][j][k]=-1;
			}
			foruser.posbl[i][j][(grid[i][j]-1)]=grid[i][j];
			foruser.np[i][j]=-5;
		}
		else{
			for(int k=0;k<9;k++){
				foruser.posbl[i][j][k]=k+1;
			}
			foruser.np[i][j]=9;
		}
	}
	}
	if(is_puzzle_symmetric==1){
		numblanks=(int)(numblanks/2); // symmetrical puzzles only
	}
	for(int a=0;a<numblanks;a++){
		int rem1,rem2; // rem2 for symmetrical puzzles only
		srand(time(NULL));
		int i;
		if(is_puzzle_symmetric==1){
			i=rand()%5; // symmetrical puzzles only
		}
		else{
			i=rand()%9;
		}
		int j=rand()%9;
		if(i==4 && j>4 && is_puzzle_symmetric==1){
			while((j=rand()%9)>4); // symmetrical puzzles only
		}
		while(1){
			if((rem1=foruser.grid[i][j])!=0){
				foruser.grid[i][j]=0;
				if(is_puzzle_symmetric==1){	
					rem2=foruser.grid[8-i][8-j]; // symmetrical puzzles only
					foruser.grid[8-i][8-j]=0; // symmetrical puzzles only
				}
				break;
			}
			else{
				if(is_puzzle_symmetric==1){
					i=rand()%5; // symmetrical puzzles only
				}
				else{
					i=rand()%9;
				}
				j=rand()%9;
				if(i==4 && j>4 && is_puzzle_symmetric==1){
					while(j=rand()%9>4); // symmetrical puzzles only
				}
			}
		}
		recursions=0,logictimes=0,correctguesses=0,failedguesses=0,multiple=0,fnflag=0;
		sudoku tmp;
		char linegrid_to_check[82];
		tmp.grid_to_line(foruser.grid,linegrid_to_check);
		tmp.init_for_solver(linegrid_to_check);
		tmp.solved();
		fnflag=1;
		if(multiple>1 || tmp.checkbasicuniqueness()==0){
			foruser.grid[i][j]=rem1; // Failed HideBlanks Operation
			if(is_puzzle_symmetric==1){
				foruser.grid[8-i][8-j]=rem2; // symmetrical puzzles only
			} // HENCE BACKTRACKING IS REQUIRED TO JUST PREVIOUS VALID GRID
			a--;
			continue;
		}
		else if(multiple<1){
			return (0);
		}
		else{ // FURTHER PROGRESSING!
			for(int k=0;k<9;k++){
				foruser.posbl[i][j][k]=k+1;
			}
			foruser.np[i][j]=9;
			if(is_puzzle_symmetric==1){
				for(int k=0;k<9;k++){
					foruser.posbl[8-i][8-j][k]=k+1; // symmetrical puzzles only
				}
				foruser.np[8-i][8-j]=9; // symmetrical puzzles only
			}
		}
	}
	recursions=0,logictimes=0,correctguesses=0,failedguesses=0,multiple=0,fnflag=0;
	sudoku tmp;
	char linegrid_to_check[82];
	tmp.grid_to_line(foruser.grid,linegrid_to_check);
	tmp.init_for_solver(linegrid_to_check);
	tmp.solved();
	fnflag=1;
	if(multiple==1 && foruser.checkbasicuniqueness()==1){
		if(recursions!=0 || no_guessing_reqd==1){
				// cout<<"Puzzle Rating: "<<(diffscore=(int)((logictimes+failedguesses*4+correctguesses*2)*numblanks/5))<<endl;
				grid_to_line(foruser.grid,question_linegrid);
				return 1;
		}
		else{ // Rejected Due To No Recursion Being Required
			return 0;
		}
	}
	else{
	return 0;
	}
}
int sudoku::randnumblanks(){
	int empty;
	srand(time(NULL));
	if(difflevel==2){
		while(1){
			if((empty=((rand()%100)/2+1))>=46){
				return empty;
			}
		}
	}
	else if(difflevel==1){
		while(1){
			if((empty=((rand()%100)/2-4))>=40){
				return empty;
			}
		}
	}
	else if(difflevel==3){
		while(1){
			if((empty=((rand()%100)/2+5))>=51){
				return empty;
			}
		}
	}
	else if(difflevel==4){
		while(1){
			if((empty=((rand()%100)/2+11))>=55){
				return empty;
			}
		}
	}
}

// solver-specific functions
void sudoku::init_for_solver(char linegrid_to_solve[82]){
	int count=0;
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		grid[i][j]=linegrid_to_solve[count++]-48;
		if(!(grid[i][j]>0 && grid[i][j]<10)){
			grid[i][j]=0;
			np[i][j]=9;
			for(int k=0;k<9;k++){
				posbl[i][j][k]=k+1;
			}
			continue;
		}
		np[i][j]=-5;
		updatedependencies(i,j,grid[i][j]-1);
	}
	}
}
void sudoku::printactivity(int i,int j,int value,int which=0){
	if(!(which==1 || which==2 || which==3)){
		cout<<"Plugged in "<<value<<" at ("<<i+1<<","<<j+1<<")";
	} // "which" is the identification no. for the logic: 1 - RCs+BOXs, 2 - NOOTHERPOSBL RC, 3 - NOOTHERPOSBL BOX
	switch(which){
		case 1:
			cout<<" by Rows/Columns/Box Elimination"<<endl;
			break;
		case 2:
			cout<<" since nowhere else possible in Rows/Columns"<<endl;
			break;
		case 3:
			cout<<" since nowhere else possible in Box"<<endl;
			break;
	}
}
void sudoku::init_current_obj(){
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		grid[i][j]=0;
		np[i][j]=9;
			for(int k=0;k<9;k++){
				posbl[i][j][k]=k+1;
			}
	}
	}
	for(int count=0;count<81;count++){
		question_linegrid[count]=48;
		solution_linegrid[count]=48;
	}
	question_linegrid[81]='\0';
	solution_linegrid[81]='\0';
}

// saving and loading functionality
void sudoku::line_to_grid(char linegrid[82]){
	int count=0;
	for (int i=0;i<9;i++){
	for (int j=0;j<9;j++){
		grid[i][j]=linegrid[count++]-48;
	}
	}
}
void sudoku::grid_to_line(int grid_to_convert[9][9],char linegrid[82]){
	int count=0;
	for(int i=0;i<9;i++){
	for(int j=0;j<9;j++){
		linegrid[count++]=grid_to_convert[i][j]+48;
	}
	}
	linegrid[count/*=81*/]='\0';
}

/*******************************/
/*** Definition of Functions ***/
/*******************************/

// functions called by the GUI after some initialization
void try_to_solve(char linegrid_to_solve[82]){
	recursions=0,logictimes=0,correctguesses=0,failedguesses=0,multiple=0;
	fnflag=0;
	sudoku grid_to_solve;
	grid_to_solve.init_for_solver(linegrid_to_solve);
	if(grid_to_solve.is_pattern_unsolvable()){
		return;
	}
	grid_to_solve.solved();
}
void generate(){
	sudoku new_question;
	int randblanks=new_question.randnumblanks();
	do{
		fnflag=1;
		new_question.initrandobj();
		new_question.solved();
	}while(new_question.checkblanks()!=0);
	while(new_question.hideblanks(randblanks)==0);
}